import pygame
import pygame.rect
import pygame.mixer
import pygame.surface
import pygame.transform

import data
import util

import weapon

import random


class Player(object):
    
    def __init__(self,pos):
        super(Player,self).__init__()
        
        self.health = 100
        
        self.rect = pygame.rect.Rect((0,0),(0,0))
        self.rect.center = pos
        
        self.alive = True
        
        self.weapons = {}
        
        self.weapons['Charge'] = weapon.Weapon('Player-Charge')
        self.weapons['Charge'].owner = self
        
        self.weapons['Sonic'] = weapon.Weapon('Player-Sonic')
        self.weapons['Sonic'].owner = self
        
        self.weapons['Missile'] = {
            'Top': weapon.Weapon('Player-Missile-Top'),
            'Bottom': weapon.Weapon('Player-Missile-Bottom'),
        }
        
        self.weapons['Missile']['Top'].owner = self
        self.weapons['Missile']['Bottom'].owner = self
        
        self.image = data.loadImage('Player.png')
        
        self.direction = 'Right'
        self.mode = 'Charge'
        
        self.sprites = []
        self.frame = 0
        
        frame_rect = pygame.rect.Rect(0,0,60,50)
        
        while frame_rect.bottom <= self.image.get_height():
            while frame_rect.right <= self.image.get_width():
                self.sprites.append(self.image.subsurface(frame_rect))
                
                frame_rect.left += frame_rect.width
                
            
            frame_rect.left = 0
            frame_rect.top += frame_rect.height
            
        
        self.velocity = [0,0]
        
        self.world = None
        
    
    def drawSelf(self,dest,offset):
        draw_x,draw_y = 0,0
        
        draw_x -= offset[0]
        draw_y -= offset[1]
        
        draw_x += self.rect.centerx
        draw_y += self.rect.centery
        
        my_image = self.sprites[self.frame]
        
        draw_x -= (my_image.get_width() // 2)
        draw_y -= (my_image.get_height() // 2)
        
        if self.direction == 'Left':
            my_image = pygame.transform.flip(my_image,True,False)
            
        
        dest.blit(my_image,(draw_x,draw_y))
        
    
    def update(self,baddies):
        if inputState['Left']:
            if self.velocity[0] > -12:
                self.velocity[0] -= 2
            elif self.velocity[0] < -12:
                self.velocity[0] += 1
            
            self.direction = 'Left'
            
        elif inputState['Right']:
            if self.velocity[0] < 12:
                self.velocity[0] += 2
            elif self.velocity[0] > 12:
                self.velocity[0] -= 1
            
            self.direction = 'Right'
            
        else:
            if self.velocity[0] >= 3:
                self.velocity[0] -= 3
            elif self.velocity[0] <= -3:
                self.velocity[0] += 3
            else:
                self.velocity[0] = 0
            
        
        if inputState['Up']:
            if self.velocity[1] > -6:
                self.velocity[1] -= 2
            elif self.velocity[1] < -6:
                self.velocity[1] += 1
            
        elif inputState['Down']:
            if self.velocity[1] < 6:
                self.velocity[1] += 2
            elif self.velocity[1] > 6:
                self.velocity[1] -= 1
            
        else:
            if self.velocity[1] >= 3:
                self.velocity[1] -= 3
            elif self.velocity[1] <= -3:
                self.velocity[1] += 3
            else:
                self.velocity[1] = 0
        
        if inputState['Mutate-Charge']:
            self.mode = 'Charge'
            
        elif inputState['Mutate-Missile']:
            self.mode = 'Missile'
            
        elif inputState['Mutate-Sonic']:
            self.mode = 'Sonic'
            
        
        self.rect.left += self.velocity[0]
        self.rect.top += self.velocity[1]
        
        if self.mode == 'Charge' and self.frame != 0:
            if self.frame == 9:
                # In neutral form
                self.frame = 2
            elif self.frame in (1,2):
                # Between neutral form and
                # charge form
                self.frame -= 1
            elif self.frame in (3,4,6,7):
                # Between other forms and
                # neutral form
                self.frame += 1
            else:
                # Almost to neutral form
                self.frame = 9
            
        elif self.mode == 'Missile' and self.frame != 3:
            if self.frame == 9:
                # In neutral form
                self.frame = 5
            elif self.frame in (4,5):
                # Between neutral form and
                # missile form
                self.frame -= 1
            elif self.frame in (0,1,6,7):
                # Between other forms and
                # neutral form
                self.frame += 1
            else:
                # Almost to neutral form
                self.frame = 9
                
            
        elif self.mode == 'Sonic' and self.frame != 6:
            if self.frame == 9:
                # In neutral form
                self.frame = 8
            elif self.frame in (7,8):
                # Between neutral form and
                # sonic form
                self.frame -= 1
            elif self.frame in (0,1,3,4):
                # Between other forms and
                # neutral form
                self.frame += 1
            else:
                # Almost to neutral form
                self.frame = 9
                
            
        
        self.weapons['Charge'].update()
        self.weapons['Sonic'].update()
        
        self.weapons['Missile']['Top'].update()
        self.weapons['Missile']['Bottom'].update()
        
        if self.mode == 'Charge' and self.frame == 0:
            self.weapons['Sonic'].stop()
            self.weapons['Missile']['Top'].stop()
            self.weapons['Missile']['Bottom'].stop()
            
            if inputState['Fire']:
                self.weapons['Charge'].start()
                
            else:
                self.weapons['Charge'].stop()
                
            
        elif self.mode == 'Missile' and self.frame == 3:
            self.weapons['Charge'].stop()
            self.weapons['Sonic'].stop()
            
            if inputState['Fire']:
                self.weapons['Missile']['Top'].start()
                self.weapons['Missile']['Bottom'].start()
                
            else:
                self.weapons['Missile']['Top'].stop()
                self.weapons['Missile']['Bottom'].stop()
                
            
        elif self.mode == 'Sonic' and self.frame == 6:
            self.weapons['Charge'].stop()
            self.weapons['Missile']['Top'].stop()
            self.weapons['Missile']['Bottom'].stop()
            
            if inputState['Fire']:
                self.weapons['Sonic'].start()
                
            else:
                self.weapons['Sonic'].stop()
                
            
        
    

class Baddie(object):
    
    def __init__(self,pos,type_name):
        super(Baddie,self).__init__()
        
        self.baddieType = str(type_name)
        
        self.rect = pygame.rect.Rect((0,0),(0,0))
        self.rect.center = pos
        
        self.image = None
        self.health = 0
        
        self.alive = True
        
        self.weapons = []
        
        self.aiState = {}
        
        self.direction = 'Right'
        
        src_image = data.loadImage('Baddie.png')
        
        if type_name == 'Goblin':
            self.image = src_image.subsurface(((0,0),(60,50)))
            
            self.health = 30
            
            self.rect.size = (34,26)
            
            self.aiState['Goal Offset'] = [
                random.randint(-60,60),
                random.randint(-50,50),
            ]
            
        elif type_name == 'Bruiser':
            self.image = src_image.subsurface(((60,0),(60,50)))
            
            self.health = 250
            
            self.rect.size = (54,42)
            
            self.aiState['Goal Offset'] = [
                random.randint(-50,50),
                random.randint(-75,75),
            ]
            
        elif type_name == 'Bomber':
            self.image = src_image.subsurface((120,0),(60,50))
            
            self.health = 100
            
            self.rect.size = (40,44)
            
            self.aiState['Turn Timer'] = 0
            self.aiState['Turn Delay'] = random.randint(40,120)
            
        else:
            raise ValueError('Invalid baddie type '+self.baddieType)
            
        
        self.baddieType = type_name
        
        self.velocity = [0,0]
        
        self.world = None
        
    
    def drawSelf(self,dest,offset):
        draw_x,draw_y = 0,0
        
        draw_x -= offset[0]
        draw_y -= offset[1]
        
        draw_x += self.rect.centerx
        draw_y += self.rect.centery
        
        draw_image = self.image
        
        if self.direction == 'Left':
            draw_image = pygame.transform.flip(draw_image,True,False)
            
        
        draw_x -= (draw_image.get_width() // 2)
        draw_y -= (draw_image.get_height() // 2)
        
        dest.blit(draw_image,(draw_x,draw_y))
        
    
    def update(self):
        if not self.alive:
            return
        
        if self.health <= 0:
            sound_name = 'Baddie-'+self.baddieType+'-Death.ogg'
            
            self.alive = False
            
        
        self.rect.left += self.velocity[0]
        self.rect.top += self.velocity[1]
        
        player = self.world.player
        
        if self.baddieType == 'Goblin':
            if player.rect.centerx - self.rect.centerx < 0:
                self.direction = 'Left'
                
            elif player.rect.centerx - self.rect.centerx > 0:
                self.direction = 'Right'
                
            
            if abs(player.rect.centerx - self.rect.centerx) > (200 + self.aiState['Goal Offset'][0]):
                if self.direction == 'Left':
                    if self.velocity[0] > -8:
                        self.velocity[0] -= 1
                    else:
                        self.velocity[0] = -8
                    
                else:
                    if self.velocity[0] < 8:
                        self.velocity[0] += 1
                    else:
                        self.velocity[0] = 8
                    
                
            else:
                if self.velocity[0] < -2:
                    self.velocity[0] += 2
                elif self.velocity[0] > 2:
                    self.velocity[0] -= 2
                else:
                    self.velocity[0] = 0
                
            
            if abs(player.rect.centery - self.rect.centery) > (25 + self.aiState['Goal Offset'][1]):
                if (player.rect.centery - self.rect.centery) < 0:
                    if self.velocity[1] > -2:
                        self.velocity[1] -= 1
                    else:
                        self.velocity[1] = -2
                    
                if (player.rect.centery - self.rect.centery) > 0:
                    if self.velocity[1] < 2:
                        self.velocity[1] += 1
                    else:
                        self.velocity[1] = 2
                    
                
            else:
                if self.velocity[1] <= -1:
                    self.velocity[1] += 1
                elif self.velocity[1] >= 1:
                    self.velocity[1] -= 1
                else:
                    self.velocity[1] = 0
            
            self.aiState['Goal Offset'][0] += random.choice([-30,-20,-10,10,20,30])
            self.aiState['Goal Offset'][1] += random.choice([-50,-30,-20,-10,10,20,30,50])
            
        elif self.baddieType == 'Bruiser':
            if player.rect.centerx - self.rect.centerx < 0:
                self.direction = 'Left'
                
            elif player.rect.centerx - self.rect.centerx > 0:
                self.direction = 'Right'
                
            
            if abs(player.rect.centerx - self.rect.centerx) > (250 + self.aiState['Goal Offset'][0]):
                if self.direction == 'Left':
                    if self.velocity[0] > -4:
                        self.velocity[0] -= 1
                    else:
                        self.velocity[0] = -4
                    
                else:
                    if self.velocity[0] < 4:
                        self.velocity[0] += 1
                    else:
                        self.velocity[0] = 4
                    
                
            else:
                if self.velocity[0] < -2:
                    self.velocity[0] += 2
                elif self.velocity[0] > 2:
                    self.velocity[0] -= 2
                else:
                    self.velocity[0] = 0
                
            
            if abs(player.rect.centery - self.rect.centery) > (25 + self.aiState['Goal Offset'][1]):
                if (player.rect.centery - self.rect.centery) < 0:
                    if self.velocity[1] > -2:
                        self.velocity[1] -= 1
                    else:
                        self.velocity[1] = -2
                    
                if (player.rect.centery - self.rect.centery) > 0:
                    if self.velocity[1] < 2:
                        self.velocity[1] += 1
                    else:
                        self.velocity[1] = 2
                    
                
            else:
                if self.velocity[1] <= -1:
                    self.velocity[1] += 1
                elif self.velocity[1] >= 1:
                    self.velocity[1] -= 1
                else:
                    self.velocity[1] = 0
            
            self.aiState['Goal Offset'][0] += random.choice([-30,-20,-10,10,20,30])
            self.aiState['Goal Offset'][1] += random.choice([-50,-30,-20,-10,10,20,30,50])
            
        elif self.baddieType == 'Bomber':
            if self.aiState['Turn Timer'] >= self.aiState['Turn Delay']:
                self.aiState['Turn Timer'] = 1
                self.aiState['Turn Delay'] = random.randint(40,120)
                
                if self.direction == 'Left':
                    self.direction = 'Right'
                else:
                    self.direction = 'Left'
                
            else:
                self.aiState['Turn Timer'] += 1
            
            if self.direction == 'Left':
                if self.velocity[0] > -6:
                    self.velocity[0] -= 1
                else:
                    self.velocity[0] = -6
                
            else:
                if self.velocity[0] < 6:
                    self.velocity[0] += 1
                else:
                    self.velocity[0] = 6
                
            
        
    


def init():
    data.loadImage('Player.png')
    data.loadImage('Baddie.png')
    
    # TODO - Pre-load the player death explosion sheet
    
    # TODO - Pre-load the "Goblin" baddie death explosion sheet
    # TODO - Pre-load the "Bruiser" baddie death explosion sheet
    # TODO - Pre-load the "Bomber" baddie death explosion sheet
    
    # TODO - Pre-load the player death sound
    
    # TODO - Pre-load the "Goblin" baddie death sound
    # TODO - Pre-load the "Bruiser" baddie death sound
    # TODO - Pre-load the "Bomber" baddie death sound
    
