import pygame
import pygame.rect
import pygame.mixer
import pygame.surface
import pygame.transform

import util
import data

class Explosion(object):
    
    def __init__(self,pos,type_name):
        super(Explosion,self).__init__()
        
        self.explosionType = str(type_name)
        
        self.image = None
        self.rect = pygame.rect.Rect((0,0),(0,0))
        self.rect.center = pos
        
        self.sprites = []
        self.frame = 0
        frame_rect = pygame.rect.Rect((0,0),(0,0))
        
        if 'Death' in self.explosionType:
            frame_rect.size = (0,0) # TODO
            
            self.rect.size = (2,2)
            
            self.damage = 0
            
        elif 'Missile' in self.explosionType:
            frame_rect.size = (60,60)
            
            self.rect.size = (40,40)
            
            self.damage = 60
            
            self.image = data.loadImage('Player-Missile-Explosion.png')
            
            while frame_rect.right <= self.image.get_width():
                self.sprites.append(self.image.subsurface(frame_rect))
                
                frame_rect.left += frame_rect.width
                
            
        
        self.maxFrame = len(self.sprites)-1
        
        self.rect.center = pos
        
        self.entitiesHit = []
        
        self.alive = True
        
        self.world = None
        
    
    def drawSelf(self,dest,offset):
        if not self.alive:
            return
        
        draw_x,draw_y = 0,0
        
        draw_x -= offset[0]
        draw_y -= offset[1]
        
        draw_x += self.rect.centerx
        draw_y += self.rect.centery
        
        my_image = self.sprites[self.frame]
        
        draw_x -= (my_image.get_width() // 2)
        draw_y -= (my_image.get_height() // 2)
        
        dest.blit(my_image,(draw_x,draw_y))
        
    
    def update(self,targets=None):
        targets = targets or []
        
        self.frame += 1
        
        if self.frame > self.maxFrame:
            self.alive = False
            return
            
        
        for targ in targets:
            if targ in self.entitiesHit:
                continue
                
            
            if self.rect.colliderect(targ.rect):
                self.entitiesHit.append(targ)
                
                targ.health -= self.damage
                
            
        
    

class Projectile(object):
    
    def __init__(self,pos,vel,size,damage,image,type_name):
        super(Projectile,self).__init__()
        
        self.projectileType = str(type_name)
        
        self.rect = pygame.rect.Rect((0,0),size)
        self.rect.center = pos
        
        self.damage = int(damage)
        
        self.image = image
        
        self.alive = True
        
        self.velocity = list(vel)
        
        self.entitiesHit = []
        
        self.world = None
        
    
    def drawSelf(self,dest,offset):
        draw_x,draw_y = 0,0
        
        draw_x -= offset[0]
        draw_y -= offset[1]
        
        draw_x += self.rect.centerx
        draw_y += self.rect.centery
        
        my_image = self.image
        
        if 'Player' in self.projectileType and self.velocity[0] < 0:
            my_image = pygame.transform.flip(my_image,True,False)
            
        
        draw_x -= (my_image.get_width() // 2)
        draw_y -= (my_image.get_height() // 2)
        
        dest.blit(my_image,(draw_x,draw_y))
        
    
    def update(self,targets):
        self.rect.left += self.velocity[0]
        self.rect.top += self.velocity[1]
        
        if self.projectileType == 'Player-Missile':
            if self.velocity[0] < 0:
                if self.velocity[0] > -18:
                    self.velocity[0] -= 1
                    
                else:
                    self.velocity[0] = -18
                
            elif self.velocity[0] > 0:
                if self.velocity[0] < 18:
                    self.velocity[0] += 1
                    
                else:
                    self.velocity[0] = 18
                
            
        elif self.projectileType == 'Player-Sonic':
            self.damage -= 2
            
            if self.damage < 12:
                self.damage = 12
            
        
        for targ in targets:
            if targ in self.entitiesHit:
                continue
                
            
            if not targ.alive:
                continue
            
            if self.rect.colliderect(targ.rect):
                if self.projectileType == 'Player-Charge':
                    self.entitiesHit.append(targ)
                    
                    # TODO - Play hit sound
                    # TODO - Spawn explosion
                    
                    if self.damage >= targ.health:
                        self.alive = True
                        targ.health = 0
                    else:
                        self.alive = False
                        targ.health -= self.damage
                        
                        return
                        
                    
                elif self.projectileType == 'Player-Missile':
                    if self.rect.colliderect(targ.rect):
                        targ.health -= self.damage
                        
                        # TODO - Play hit sound
                        self.world.addExplosion(Explosion(self.rect.center,'Player-Missile'),True)
                        
                        self.alive = False
                        
                        return
                        
                    
                elif self.projectileType == 'Player-Sonic':
                    if self.rect.colliderect(targ.rect):
                        targ.health -= self.damage
                        
                        self.entitiesHit.append(targ)
                        
                        # TODO - Play hit sound
                        # TODO - Spawn explosion
                        
                        continue
                        
                    
                elif self.projectileType == 'Baddie-Shot':
                    if self.rect.colliderect(targ.rect):
                        targ.health -= self.damage
                        
                        self.entitiesHit.append(targ)
                        
                        # TODO - Play hit sound
                        # TODO - Spawn explosion
                        
                        self.alive = False
                        
                        return
                        
                    
                
            
    

class Weapon(object):
    
    def __init__(self,type_name):
        super(Weapon,self).__init__()
        
        self.weaponType = type_name
        
        self.otherData = {}
        
        self.running = False
        
        self.owner = None
        self.offset = [0,0]
        
        self.cooldownTimer = 0
        self.cooldownLimit = 0
        
        if self.weaponType == 'Player-Charge':
            self.otherData['Charge Level'] = 0
            self.otherData['Timer'] = 0
            
            self.offset[0] = 25
            self.offset[1] = 0
            
            self.cooldownLimit = 5
            
        elif 'Player-Missile' in self.weaponType:
            if 'Top' in self.weaponType:
                self.cooldownLimit = 40
                
                self.offset[0] = 10
                self.offset[1] = -15
                
            else:
                self.cooldownLimit = 30
                
                self.offset[0] = 9
                self.offset[1] = 15
                
            
        elif self.weaponType == 'Player-Sonic':
            self.cooldownLimit = 20
            
            self.offset[0] = 20
            self.offset[1] = 0
            
        elif 'Baddie' in self.weaponType:
            if 'Goblin' in self.weaponType:
                # TODO
                pass
                
            elif 'Bruiser' in self.weaponType:
                # TODO
                pass
                
            elif 'Bomber' in self.weaponType:
                # TODO
                pass
                
            
        
    
    def start(self):
        if self.running: return
        
        self.running = True
        
        if self.weaponType == 'Player-Charge':
            pass
            
        elif 'Player-Missile' in self.weaponType:
            pass
            
        elif self.weaponType == 'Player-Sonic':
            pass
            
        
    
    def stop(self):
        if not self.running: return
        
        self.running = False
        
        if self.weaponType == 'Player-Charge':
            self.fire()
            
        elif 'Player-Missile' in self.weaponType:
            pass
            
        elif self.weaponType == 'Player-Sonic':
            pass
            
        
    
    def makeProjectile(self):
        if self.weaponType == 'Player-Charge':
            damage = 0
            image = None
            img_rect = pygame.rect.Rect((0,0),(50,40))
            
            position = [0,0]
            velocity = [0,0]
            size = [0,0]
            
            position[0] += self.owner.rect.centerx
            position[1] += self.owner.rect.centery
            
            if self.owner.direction == 'Left':
                position[0] -= self.offset[0]
                position[1] += self.offset[1]
                
                velocity[0] = -16
                
            else:
                position[0] += self.offset[0]
                position[1] += self.offset[1]
                
                velocity[0] = 16
            
            src_image = data.loadImage('Player-Charge-Shot.png')
            
            if self.otherData['Charge Level'] == 0:
                img_rect.left = 0
                
                damage = 10
                size[0] = 14
                size[1] = 12
                
            elif self.otherData['Charge Level'] == 1:
                img_rect.left = 50
                
                damage = 20
                size[0] = 20
                size[1] = 14
                
            elif self.otherData['Charge Level'] == 2:
                img_rect.left = 100
                
                damage = 40
                size[0] = 24
                size[1] = 18
                
            elif self.otherData['Charge Level'] == 3:
                img_rect.left = 150
                
                damage = 60
                size[0] = 28
                size[1] = 22
                
            elif self.otherData['Charge Level'] == 4:
                img_rect.left = 200
                
                damage = 100
                size[0] = 32
                size[1] = 28
                
            
            image = src_image.subsurface(img_rect)
            
            new_proj = Projectile(position,velocity,size,damage,image,'Player-Charge')
            
            new_proj.world = self.owner.world
            
            return new_proj
            
        elif 'Player-Missile' in self.weaponType:
            damage = 150
            image = data.loadImage('Player-Missile.png')
            
            position = [0,0]
            velocity = [0,0]
            size = [22,20]
            
            position[0] += self.owner.rect.centerx
            position[1] += self.owner.rect.centery
            
            if self.owner.direction == 'Left':
                velocity[0] = -1
                velocity[1] = 0
                
                position[0] -= self.offset[0]
                position[1] += self.offset[1]
                
            else:
                velocity[0] = 1
                velocity[1] = 0
                
                position[0] += self.offset[0]
                position[1] += self.offset[1]
                
            
            new_proj = Projectile(position,velocity,size,damage,image,'Player-Missile')
            
            new_proj.world = self.owner.world
            
            return new_proj
            
        elif self.weaponType == 'Player-Sonic':
            position = [0,0]
            size = [40,30]
            velocity = [0,0]
            
            damage = 120
            
            image = data.loadImage('Player-Sonic-Wave.png')
            
            position[0] += self.owner.rect.centerx
            position[1] += self.owner.rect.centery
            
            if self.owner.direction == 'Left':
                position[0] -= self.offset[0]
                position[1] += self.offset[1]
                
                velocity[0] = -20
                velocity[1] = 0
                
            else:
                position[0] += self.offset[0]
                position[1] += self.offset[1]
                
                velocity[0] = 20
                velocity[1] = 0
                
            
            new_proj = Projectile(position,velocity,size,damage,image,'Player-Sonic')
            
            return new_proj
            
        elif 'Baddie' in self.weaponType:
            position = [0,0]
            size = [14,14]
            velocity = [0,0]
            
            damage = 20
            
            position[0] += self.owner.rect.centerx
            position[1] += self.owner.rect.centery
            
            if self.owner.direction == 'Left':
                position[0] -= self.offset[0]
                position[1] += self.offset[1]
                
            else:
                position[0] += self.offset[0]
                position[1] += self.offset[1]
                
            
            image = data.loadImage('Baddie-Shot.png')
            
            # TODO - Logic for velocity
            
            new_proj = Projectile(position,velocity,size,damage,image,'Baddie-Shot')
            
            return new_proj
            
        
    
    def fire(self):
        if self.cooldownTimer > 0:
            return
        
        self.cooldownTimer = self.cooldownLimit
        
        new_proj = self.makeProjectile()
        
        self.owner.world.addProjectile(new_proj,self.owner)
        
        
        if self.weaponType == 'Player-Charge':
            self.otherData['Charge Level'] = 0
            self.otherData['Timer'] = 0
            
        
    
    def update(self):
        self.cooldownTimer -= 1
        
        if self.cooldownTimer < 0:
            self.cooldownTimer = 0
        
        if self.cooldownTimer == 0:
            if self.running:
                if 'Charge' not in self.weaponType:
                    self.fire()
                    
                
            
        
        if self.weaponType == 'Player-Charge':
            if self.running:
                if self.otherData['Charge Level'] >= 4:
                    self.otherData['Charge Level'] = 4
                    self.otherData['Timer'] = -1
                    
                
                self.otherData['Timer'] += 1
                
                while self.otherData['Timer'] >= 10:
                    self.otherData['Timer'] -= 10
                    self.otherData['Charge Level'] += 1
                    
            
        elif 'Player-Missile' in self.weaponType:
            pass
            
        elif self.weaponType == 'Player-Sonic':
            pass
            
        
        
    


def init():
    data.loadImage('Player-Charge-Shot.png')
    data.loadImage('Player-Missile.png')
    data.loadImage('Player-Sonic-Wave.png')
    
    data.loadImage('Baddie-Shot.png')
    
    data.loadImage('Player-Charge-Explosion.png')
    data.loadImage('Player-Missile-Explosion.png')
    data.loadImage('Player-Sonic-Explosion.png')
    
    # TODO - Load baddie shot explosion sheet
    
    for lvl in range(5):
        # TODO - Pre-load each player charge shot sound
        # TODO - Pre-load each player charge hit sound
        
        break
        
    
    # TODO - Pre-load the player missile fire sound
    # TODO - Pre-load the player missile hit sound
    # TODO - Pre-load the player missile explosion hit sound
    
    # TODO - Pre-load the sonic wave fire sound
    # TODO - Pre-load the sonic wave hit sound
    
